@@ -67,6 +67,18 @@ RESPONSE_ADD = 2
     STORE_VISIBLE
 ) = range(5)
 
+def error(parent_window, summary, msg):
+  " show a error dialog "
+  dialog = gtk.MessageDialog(parent=parent_window,
+                             flags=gtk.DIALOG_MODAL,
+                             type=gtk.MESSAGE_ERROR,
+                             buttons=gtk.BUTTONS_OK,
+                             message_format=None)
+  dialog.set_markup("<big><b>%s</b></big>\n\n%s" % (summary, msg))
+  res = dialog.run()
+  dialog.destroy()
+  return False
+
 class SoftwarePropertiesGtk(SoftwareProperties,SimpleGladeApp):
   def __init__(self, datadir=None, options=None, file=None, parent=None):
     """ Provide a GTK based graphical user interface to configure
@@ -81,7 +93,10 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGladeApp):
     if parent:
       self.window_main.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_DIALOG)
       self.window_main.show()
-      self.window_main.set_transient_for(parent)
+      try:
+        self.window_main.set_transient_for(parent)
+      except:
+        pass
 
     # If externally called, reparent to external application.
     self.options = options
@@ -90,7 +105,10 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGladeApp):
       self.window_main.show()
       toplevel = gtk.gdk.window_foreign_new(int(options.toplevel))
       if (toplevel):
-      	self.window_main.window.set_transient_for(toplevel)
+        try:
+          self.window_main.window.set_transient_for(toplevel)
+        except: 
+          pass
     if options and options.open_tab:
       self.notebook_main.set_current_page(int(options.open_tab))
 
@@ -128,6 +146,11 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGladeApp):
                                            apt_pkg.Config.Find("Dir::Etc::sourcelist")):
         self.open_file(file)
 
+  def update_interface(self):
+    " abstract interface to keep the UI alive "
+    while gtk.events_pending():
+      gtk.main_iteration()
+
   def init_popcon(self):
     """ If popcon is enabled show the statistics tab and an explanation
         corresponding to the used distro """
@@ -566,10 +589,10 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGladeApp):
     self.treeview_sources.append_column(col_active)
     self.treeview_sources.append_column(col_desc)
     # drag and drop support for sources.list
-    self.treeview_sources.drag_dest_set(gtk.DEST_DEFAULT_ALL, \
-                                        [('text/uri-list',0, 0)], \
+    self.treeview_sources.drag_dest_set(gtk.DEST_DEFAULT_ALL,
+                                        [('text/uri-list',0, 0)],
                                         gtk.gdk.ACTION_COPY)
-    self.treeview_sources.connect("drag_data_received",\
+    self.treeview_sources.connect("drag_data_received",
                                   self.on_sources_drag_data_received)
 
   def on_isv_source_activate(self, treeview, path, column):
@@ -597,10 +620,48 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGladeApp):
   def init_keys(self):
     """Setup the user interface parts needed for the key handling"""
     self.keys_store = gtk.ListStore(str)
-    self.treeview2.set_model(self.keys_store)
+    self.treeview_auth.set_model(self.keys_store)
     tr = gtk.CellRendererText()
     keys_col = gtk.TreeViewColumn("Key", tr, text=0)
-    self.treeview2.append_column(keys_col)
+    self.treeview_auth.append_column(keys_col)
+    self.treeview_auth.enable_model_drag_dest(
+      [('text/plain', 0, 0)], gtk.gdk.ACTION_COPY)
+    self.treeview_auth.connect("drag_data_received",
+                                  self.on_auth_drag_data_received)
+    self.treeview_auth.connect("button-press-event",
+                               self.show_auth_context_menu)
+
+  def show_auth_context_menu(self, widget, event):
+    if event.type == gtk.gdk.BUTTON_PRESS and event.button == 3:
+      menu = gtk.Menu()
+      item_paste = gtk.MenuItem(_("_Add key from paste data"))
+      item_paste.connect("activate", self.on_auth_add_key_from_paste)
+      menu.add(item_paste)
+      menu.popup(None, None, None, 0, event.time)
+      menu.show_all()
+      return True
+
+  def on_auth_add_key_from_paste(self, widget):
+    keydata = gtk.Clipboard().wait_for_text()
+    if not keydata:
+      return
+    if not self.add_key_from_data(keydata):
+      error(self.window_main,
+            _("Error importing key"),
+            _("The selected data may not be a GPG key file "
+              "or it might be corrupt."))
+    self.show_keys()
+
+  def on_auth_drag_data_received(self, widget, context, x, y,
+                                 selection, target_type, timestamp):
+      """Extract the dropped key and add it to the keyring"""
+      keydata = selection.data.strip()
+      if not self.add_key_from_data(keydata):
+        error(self.window_main,
+              _("Error importing key"),
+              _("The selected data may not be a GPG key file "
+                "or it might be corrupt."))
+      self.show_keys()
 
   #FIXME revert automation settings too
   def on_button_revert_clicked(self, button):
@@ -697,7 +758,7 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGladeApp):
     if line != None:
       self.add_source_from_line(line)
       self.set_modified_sourceslist()
-      
+
   def on_edit_clicked(self, widget):
     """Show a dialog to edit an ISV source"""
     sel = self.treeview_sources.get_selection()
@@ -754,6 +815,9 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGladeApp):
                                     buttons=(gtk.STOCK_CANCEL,
                                              gtk.RESPONSE_REJECT,
                                              gtk.STOCK_OK,gtk.RESPONSE_ACCEPT))
+    if "SUDO_USER" in os.environ:
+        home = os.path.expanduser("~%s" % os.environ["SUDO_USER"])
+        chooser.set_current_folder(home)
     res = chooser.run()
     chooser.hide()
     if res == gtk.RESPONSE_ACCEPT:
@@ -766,7 +830,7 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGladeApp):
 
   def remove_key_clicked(self, widget):
     """Remove a trusted software vendor key"""
-    selection = self.treeview2.get_selection()
+    selection = self.treeview_auth.get_selection()
     (model,a_iter) = selection.get_selected()
     if a_iter == None:
         return
@@ -812,20 +876,13 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGladeApp):
     try:
       res = cdrom.Add(progress)
     except SystemError, msg:
-      #print "aiiiieeee, exception from cdrom.Add() [%s]" % msg
-      progress.close()
-      dialog = gtk.MessageDialog(parent=self.window_main,
-                                 flags=gtk.DIALOG_MODAL,
-                                 type=gtk.MESSAGE_ERROR,
-                                 buttons=gtk.BUTTONS_OK,
-                                 message_format=None)
-      dialog.set_markup(_("<big><b>Error scanning the CD</b></big>\n\n%s")%msg)
-      res = dialog.run()
-      dialog.destroy()
+      error(self.window_main, _("Error scanning the CD"), msg)
       return
-    apt_pkg.Config.Set("Dir::Etc::sourcelist",saved_entry)
-    if res == False:
+    finally:
+      apt_pkg.Config.Set("Dir::Etc::sourcelist",saved_entry)
       progress.close()
+
+    if res == False:
       return
     # read tmp file with source name (read only last line)
     line = ""
@@ -833,5 +890,6 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGladeApp):
       line = x
     if line != "":
       full_path = "%s%s" % (apt_pkg.Config.FindDir("Dir::Etc"),saved_entry)
-      self.sourceslist.list.append(SourceEntry(line,full_path))
+      # insert cdrom source first, so that it has precedence over network sources
+      self.sourceslist.list.insert(0, SourceEntry(line,full_path))
       self.set_modified_sourceslist()
