@@ -1,8 +1,9 @@
-#!/usr/bin/python
 # -*- coding: utf-8 -*-
+#
 #  Qt 4 based frontend to software-properties
 #
-#  Copyright (c) 2007 Canonical Ltd.
+#  Copyright © 2009 Harald Sitter <apachelogger@ubuntu.com>
+#  Copyright © 2007 Canonical Ltd.
 #
 #  Author: Jonathan Riddell <jriddell@ubuntu.com>
 #
@@ -37,7 +38,7 @@ from PyQt4.QtCore import *
 from PyQt4.QtGui import *
 from PyQt4 import uic
 
-from PyKDE4.kdecore import ki18n, KUrl
+from PyKDE4.kdecore import ki18n, i18n, KUrl
 from PyKDE4.kdeui import KApplication, KMessageBox, KIcon, KWindowSystem
 from PyKDE4.kio import KFileDialog
 
@@ -45,22 +46,18 @@ import softwareproperties
 import softwareproperties.distro
 from softwareproperties.SoftwareProperties import SoftwareProperties
 import softwareproperties.SoftwareProperties
+from I18nHelper import *
+
 from aptsources.sourceslist import SourceEntry
 from DialogEdit import DialogEdit
+from DialogAdd import DialogAdd
 from DialogMirror import DialogMirror
 from CdromProgress import CdromProgress
 
-def utf8(str):
-  " helper that takes a string and converts it to utf8 unicode "
-  if isinstance(str, unicode):
-      return str
-  return unicode(str, 'UTF-8')
-
 class SoftwarePropertiesKDEUI(QWidget):
-
-    def __init__(self, datadir):
-        QWidget.__init__(self)
-        uic.loadUi("%s/designer/main.ui" % datadir, self)
+  def __init__(self, datadir):
+      QWidget.__init__(self)
+      uic.loadUi("%s/designer/main.ui" % datadir, self)
 
 class SoftwarePropertiesKDE(SoftwareProperties):
   def __init__(self, datadir=None, options=None, parent=None, file=None, attachWinID=None):
@@ -84,7 +81,7 @@ class SoftwarePropertiesKDE(SoftwareProperties):
     self.userinterface.button_edit.setIcon(KIcon("document-edit"))
     self.userinterface.button_add.setIcon(KIcon("list-add"))
     self.userinterface.button_add_cdrom.setIcon(KIcon("media-optical"))
-    self.translate_widgets()
+    translate_widget(self.userinterface)
     self.userinterface.show()
     if attachWinID is not None:
         KWindowSystem.setMainWindow(self.userinterface, int(attachWinID))
@@ -119,8 +116,8 @@ class SoftwarePropertiesKDE(SoftwareProperties):
     kapp.connect(self.userinterface.button_add_cdrom, SIGNAL("clicked()"), self.on_button_add_cdrom_clicked)
     kapp.connect(self.userinterface.button_add, SIGNAL("clicked()"), self.on_add_clicked)
     kapp.connect(self.userinterface.treeview_sources, SIGNAL("itemChanged(QTreeWidgetItem*, int)"), self.on_isv_source_toggled)
-    kapp.connect(self.userinterface.treeview_cdroms, SIGNAL("itemChanged(QTreeWidgetItem*, int)"), self.on_cdrom_source_toggled)
     kapp.connect(self.userinterface.treeview_sources, SIGNAL("itemClicked(QTreeWidgetItem*, int)"), self.on_treeview_sources_cursor_changed)
+    kapp.connect(self.userinterface.treeview_cdroms, SIGNAL("itemChanged(QTreeWidgetItem*, int)"), self.on_cdrom_source_toggled)
     kapp.connect(self.userinterface.treeview2, SIGNAL("itemClicked(QTreeWidgetItem*, int)"), self.on_treeview_keys_cursor_changed)
     button_close = self.userinterface.buttonBox.button(QDialogButtonBox.Close)
     button_close.setIcon(KIcon("dialog-close"))
@@ -131,39 +128,6 @@ class SoftwarePropertiesKDE(SoftwareProperties):
     self.init_distro()
     self.show_distro()
 
-  def translate_widgets(self):
-      """translate the whole user interface"""
-      self.translate_widget_children(self.userinterface)
-
-  def translate_widget_children(self, parentWidget):
-      """translate given widget and its children"""
-      for widget in parentWidget.children():
-          self.translate_widget(widget)
-          self.translate_widget_children(widget)
-
-  def translate_widget(self, widget):
-      """translate the text on a widget"""
-      if isinstance(widget, QLabel):
-          text = str(widget.text())
-          text = _(text)
-          text = utf8(text)
-          widget.setText(text)
-      elif isinstance(widget, QPushButton):
-          text = str(widget.text())
-          text = _(text)
-          text = utf8(text)
-          widget.setText(text)
-      elif isinstance(widget, QGroupBox):
-          text = str(widget.title())
-          text = _(text)
-          text = utf8(text)
-          widget.setTitle(text)
-      elif isinstance(widget, QCheckBox):
-          text = str(widget.text())
-          text = _(text)
-          text = utf8(text)
-          widget.setText(text)
-
   def init_popcon(self):
     """ If popcon is enabled show the statistics tab and an explanation
         corresponding to the used distro """
@@ -242,12 +206,11 @@ class SoftwarePropertiesKDE(SoftwareProperties):
 
   def init_distro(self):
     # TRANS: %s stands for the distribution name e.g. Debian or Ubuntu
-    text = _("%s updates") % self.distro.id
+    text = u_("%s updates") % self.distro.id
     text = text.replace("Ubuntu", "Kubuntu")
-    self.userinterface.groupBox_updates.setTitle(utf8(text))
+    self.userinterface.groupBox_updates.setTitle(text)
     # TRANS: %s stands for the distribution name e.g. Debian or Ubuntu
-    text = _("%s Software") % self.distro.id
-    text = utf8(text)
+    text = u_("%s Software") % self.distro.id
     text = text.replace("Ubuntu", "Kubuntu")
     self.userinterface.tabWidget.setTabText(0, text)
 
@@ -299,10 +262,15 @@ class SoftwarePropertiesKDE(SoftwareProperties):
     else:
         self.userinterface.vbox_updates_frame.setEnabled(True)
 
+    self.init_mirrors()
+    kapp.connect(self.userinterface.combobox_server, SIGNAL("activated(int)"), self.on_combobox_server_changed)
+
+  def init_mirrors(self):
     # Intiate the combobox which allows the user to specify a server for all
     # distro related sources
     seen_server_new = []
     self.mirror_urls = []
+    self.userinterface.combobox_server.clear()
     for (name, uri, active) in self.distro.get_server_list():
         ##server_store.append([name, uri, False])
         self.userinterface.combobox_server.addItem(utf8(name))
@@ -316,13 +284,12 @@ class SoftwarePropertiesKDE(SoftwareProperties):
             self.userinterface.combobox_server.setCurrentIndex(self.userinterface.combobox_server.count() - 1)
     for [name, uri] in self.seen_server:
         self.userinterface.combobox_server.addItem(utf8(name))
+        self.mirror_urls.append(uri)
     self.seen_server = seen_server_new
     # add a separator and the option to choose another mirror from the list
     ##FIXME server_store.append(["sep", None, True])
     self.userinterface.combobox_server.addItem(utf8(_("Other...")))
-    self.mirror_urls.append("")
-
-    kapp.connect(self.userinterface.combobox_server, SIGNAL("currentIndexChanged(int)"), self.on_combobox_server_changed)
+    self.other_mirrors_index = self.userinterface.combobox_server.count() - 1
 
   def show_distro(self):
     """
@@ -420,26 +387,26 @@ class SoftwarePropertiesKDE(SoftwareProperties):
     the selected one
     """
     combobox = self.userinterface.combobox_server
-    if combobox.currentIndex() == self.active_server:
+    index = combobox.currentIndex()
+    if index == self.active_server:
         return
-    uri = self.mirror_urls[combobox.currentIndex()]
-    name = utf8(combobox.currentText())
-    if name == utf8(_("Other...")):
+    elif index == self.other_mirrors_index:
         dialogue = DialogMirror(self.userinterface, self.datadir, self.distro, self.custom_mirrors)
-        self.translate_widget_children(dialogue)
         res = dialogue.run()
         if res != None:
             self.distro.change_server(res)
             self.set_modified_sourceslist()
-            ##FIXME insert new URL into combo box
+            self.init_mirrors() # update combobox
         else:
             combobox.setCurrentIndex(self.active_server)
-    elif uri != None and len(self.distro.used_servers) > 0:
-        self.active_server = combobox.currentIndex()
-        self.distro.change_server(uri)
-        self.set_modified_sourceslist()
     else:
-        self.distro.default_server = uri
+        uri = self.mirror_urls[index]
+        if uri != None and len(self.distro.used_servers) > 0:
+            self.active_server = index
+            self.distro.change_server(uri)
+            self.set_modified_sourceslist()
+        else:
+            self.distro.default_server = uri
 
   def on_component_toggled(self):
     """
@@ -516,15 +483,23 @@ class SoftwarePropertiesKDE(SoftwareProperties):
 
   def on_cdrom_source_toggled(self, item):
     """Enable or disable the selected channel"""
-    index = self.userinterface.treeview_cdroms.indexOfTopLevelItem(item)
-    source_entry = self.cdrom_sources[index]
-    self.toggle_source_use(source_entry)
+    self.toggle_source(self.cdrom_sources, self.userinterface.treeview_cdroms, item)
 
   def on_isv_source_toggled(self, item):
     """Enable or disable the selected channel"""
-    index = self.userinterface.treeview_sources.indexOfTopLevelItem(item)
-    source_entry = self.isv_sources[index]
+    self.toggle_source(self.isv_sources, self.userinterface.treeview_sources, item)
+
+  def toggle_source(self, sources, treeview, item):
+    """Enable or disable the selected channel"""
+    index = treeview.indexOfTopLevelItem(item)
+    source_entry = sources[index]
     self.toggle_source_use(source_entry)
+    item = treeview.topLevelItem(index) # old item was destroyed
+    if item != 0:
+      treeview.setCurrentItem(item) # reselect entry after refresh
+    else:
+      self.userinterface.button_remove.setEnabled(False)
+      self.userinterface.button_edit.setEnabled(False)
 
   def init_keys(self):
     """Setup the user interface parts needed for the key handling"""
@@ -563,10 +538,26 @@ class SoftwarePropertiesKDE(SoftwareProperties):
   def show_isv_sources(self):
     """ Show the repositories of independent software vendors in the
         third-party software tree view """
-    self.userinterface.treeview_sources.clear()
-    self.isv_sources = []
+    self.isv_sources = self.get_isv_sources()[:]
+    self.show_sources(self.isv_sources, self.userinterface.treeview_sources)
 
-    for source in self.get_isv_sources():
+    if self.isv_sources < 1:
+        self.userinterface.button_remove.setEnabled(False)
+        self.userinterface.button_edit.setEnabled(False)
+
+  def show_cdrom_sources(self):
+    """ Show CD-ROM/DVD based repositories of the currently used distro in
+        the CDROM based sources list """
+    self.cdrom_sources = self.get_cdrom_sources()[:]
+    self.show_sources(self.cdrom_sources, self.userinterface.treeview_cdroms)
+
+  def show_sources(self, sources, treeview):
+    # this workaround purposely replaces treeview.clear() (LP #102792)
+    while treeview.topLevelItemCount() > 0:
+      treeview.takeTopLevelItem(0)
+
+    items = []
+    for source in sources:
         contents = self.render_source(source)
         contents = contents.replace("<b>", "")
         contents = contents.replace("</b>", "")
@@ -575,31 +566,9 @@ class SoftwarePropertiesKDE(SoftwareProperties):
             item.setCheckState(0, Qt.Checked)
         else:
             item.setCheckState(0, Qt.Unchecked)
-        self.userinterface.treeview_sources.addTopLevelItem(item)
-        self.isv_sources.append(source)
+        items.append(item)
+    treeview.addTopLevelItems(items)
 
-    if self.userinterface.treeview_sources.topLevelItemCount() < 1:
-        self.userinterface.button_remove.setEnabled(False)
-        self.userinterface.button_edit.setEnabled(False)
-
-  def show_cdrom_sources(self):
-    """ Show CD-ROM/DVD based repositories of the currently used distro in
-        the CDROM based sources list """
-    self.userinterface.treeview_cdroms.clear()
-    self.cdrom_sources = []
-
-    for source in self.get_cdrom_sources():
-        contents = self.render_source(source)
-        contents = contents.replace("<b>", "")
-        contents = contents.replace("</b>", "")
-        item = QTreeWidgetItem([contents])
-        if not source.disabled:
-            item.setCheckState(0, Qt.Checked)
-        else:
-            item.setCheckState(0, Qt.Unchecked)
-        self.userinterface.treeview_cdroms.addTopLevelItem(item)
-        self.cdrom_sources.append(source)
-    
   def show_keys(self):
     self.userinterface.treeview2.clear()
     for key in self.apt_key.list():
@@ -642,20 +611,12 @@ class SoftwarePropertiesKDE(SoftwareProperties):
 
   def on_add_clicked(self):
     """Show a dialog that allows to enter the apt line of a to be used repo"""
-    if self.distro:
-        example = "%s %s %s %s" % (self.distro.binary_type,
-                                   self.distro.source_template.base_uri,
-                                   self.distro.codename,
-                                   self.distro.source_template.components[0].name)
-    else:
-        example = "deb http://ftp.debian.org sarge main"
-    # L10N: the example is of the format: deb http://ftp.debian.org sarge main
-    text = _('<p><b>Enter the complete APT line of the<br /> repository that you want to add as source</b></p><p>The APT line includes the type, location and components of a repository,<br /> for example <i>"%s"</i>.</p>' % example)
-    (line, valid) = QInputDialog.getText(self.userinterface, utf8(_("Add APT repository")), text)
-    uline = utf8(line)
-    if valid:
-        self.add_source_from_line(uline)
-        self.show_isv_sources()
+    dialog = DialogAdd(self.userinterface, self.sourceslist,
+                       self.datadir, self.distro)
+    line = dialog.run()
+    if line != None:
+      self.add_source_from_line(utf8(line))
+      self.set_modified_sourceslist()
 
   def on_edit_clicked(self):
     """Show a dialog to edit an ISV source"""
@@ -663,7 +624,6 @@ class SoftwarePropertiesKDE(SoftwareProperties):
     if item is not None:
         index = self.userinterface.treeview_sources.indexOfTopLevelItem(item)
         dialogue = DialogEdit(self.userinterface, self.sourceslist, self.isv_sources[index], self.datadir)
-        self.translate_widget_children(dialogue)
         result = dialogue.run()
         if result == QDialog.Accepted:
             self.set_modified_sourceslist()
@@ -692,7 +652,7 @@ class SoftwarePropertiesKDE(SoftwareProperties):
     if "SUDO_USER" in os.environ:
         home = os.path.expanduser("~%s" % os.environ["SUDO_USER"])
     url = KUrl.fromPath(home)
-    filename = KFileDialog.getOpenFileName(url, '*.asc *.pkr *.skr *.gpg *.gpgkey| PGP keys', self.userinterface, utf8(_("Import key")))
+    filename = KFileDialog.getOpenFileName(url, 'application/pgp-keys', self.userinterface, utf8(_("Import key")))
     if filename:
       if not self.add_key(filename):
         title = _("Error importing selected file")
@@ -758,14 +718,15 @@ class SoftwarePropertiesKDE(SoftwareProperties):
     try:
       res = cdrom.Add(progress)
     except SystemError, msg:
-      progress.close()
       title = _("CD Error")
       text = _("<big><b>Error scanning the CD</b></big>\n\n%s")%msg
       KMessageBox.sorry(self.userinterface, utf8(text), utf8(title))
       return
-    apt_pkg.Config.Set("Dir::Etc::sourcelist",saved_entry)
-    if res == False:
+    finally:
+      apt_pkg.Config.Set("Dir::Etc::sourcelist",saved_entry)
       progress.close()
+
+    if res == False:
       return
     # read tmp file with source name (read only last line)
     line = ""
@@ -773,7 +734,8 @@ class SoftwarePropertiesKDE(SoftwareProperties):
       line = x
     if line != "":
       full_path = "%s%s" % (apt_pkg.Config.FindDir("Dir::Etc"),saved_entry)
-      self.sourceslist.list.append(SourceEntry(line,full_path))
+      # insert cdrom source first, so that it has precedence over network sources
+      self.sourceslist.list.insert(0, SourceEntry(line,full_path))
       self.set_modified_sourceslist()
 
   def run(self):
